//###########################################################################//
//
//Title :  Example 2 of 3d Programming 
//
//Author:  David Coz      
//
//Mail  :  coz.hubert@infonie.fr
//
// Rotate a Cube Around One Axis without Lookup Tables (Slow) 
//
//Created 13/04/2002; 00:21:07
//
//############################################################################//




#define USE_TI92PLUS              

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files



//Structure for Vertex (3D point)
typedef struct
{

    int x;
    int y;
    int z;
  
} Vertex;


//Structure fo Screen Point(2D point)
typedef struct
{

    int x;
    int y;
      
} ScrPoint;




// Main Function
void _main(void)
{

//Dissable all interrupts
OSSetSR(0x0400);

//Virtual Screen
void *Vscreen = malloc(3840);

//Graphical functions (like DrawLine) redirected to Vscreen...
PortSet(Vscreen,239,127);


//define the Screen Center
ScrPoint Screen_center = { 120 , 64 };

//define the perspective Distorsion
int Distorsion = 120 ;

// Angle Around the X-Axis in Radian
float ang_x = 0;

int p,l,intermediate;

//distance between the Camera and the Cube
int dist = 50;

//vertices definition of the cube 
Vertex CUBE [8] = { {-10,-10,-10},{10,-10,-10},{10,10,-10},{-10,10,-10},
                     {-10,-10,10}, {10,-10,10}, {10,10,10},{-10,10,10} };

//It will represent the Vertices on the Screen
ScrPoint SP_CUBE [8] ; 

//Store Vertices after transformations
Vertex Work_CUBE ;

//Will contain the Camera Position
Vertex Camera = { 0 , 0 , 0 };

//Lines that join the Vertices
int LINES [12] [2] = { {0,1} , {1,2} , {2,3} , {3,0} , {4,5} , {5,6} , {6,7} ,
 {7,4} , {0,4} , {1,5} , {2,6} , {3,7} } ;  


//While you don't press ESC...
while( ! (_rowread(~((short)(1<<8))) & (1<<6) ))
{
	ClrScr();
	
	//Calculate Camera Position,depending on ang_x...
	Camera.y = dist * sin(ang_x);
	Camera.z = dist * cos(ang_x);
	
	
	 for (p=0 ; p<8 ;p++) // For the 8 points...
	{
		
		// Store the values in Work
		Work_CUBE.x = CUBE[p].x - Camera.x;
		Work_CUBE.y = CUBE[p].y - Camera.y;
		Work_CUBE.z = CUBE[p].z - Camera.z;
		
		intermediate = Work_CUBE.y;
		
		// Rotation Around X-Axis...
		Work_CUBE.y = -Work_CUBE.y*cos(ang_x) + Work_CUBE.z*sin(ang_x);
		Work_CUBE.z = Work_CUBE.z*cos(ang_x) + intermediate*sin(ang_x);
				
		
		// Calculate the ScrPoints...
		SP_CUBE[p].x = (Work_CUBE.x*Distorsion)/Work_CUBE.z + Screen_center.x;
		SP_CUBE[p].y = (Work_CUBE.y*Distorsion)/Work_CUBE.z + Screen_center.y;
		
	}
	
	
	
	//Now that we have the ScrPoints,we must draw the lines on the screen...
	
	
	int P1_x , P2_x , P1_y , P2_y ;// Intermediate values...
	
	 for (l=0 ; l<12 ;l++)  // For the 12 lines...
	{
			P1_x = SP_CUBE [ LINES [l][0] ].x ; // First Point x value...
			P1_y = SP_CUBE [ LINES [l][0] ].y ; // First Point y value...
			P2_x = SP_CUBE [ LINES [l][1] ].x ; // Second Point x value...
			P2_y = SP_CUBE [ LINES [l][1] ].y ; // Second Point y value...
			
			//Now draw the line between the 2 points...
			DrawLine ( P1_x , P1_y , P2_x , P2_y , A_NORMAL );
			 
	}
	// Add PI/24 to Ang_x  ( 7.5 Degrees ) 
	ang_x += PI/24;

	
	//Display the Virtual Screen...
	memcpy(LCD_MEM,Vscreen,3840);
	
}

free(Vscreen);
PortRestore();

}


//  by David Coz      coz.hubert@infonie.fr
